# mandayejs - saml reverse proxy
# Copyright (C) 2015  Entr'ouvert
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from __future__ import absolute_import

import logging
import requests

from django.contrib.auth.models import User
from rest_framework import status
from rest_framework.views import APIView
from rest_framework.response import Response

from mandayejs.mandaye.models import UserCredentials
from mandayejs.applications import get_app_settings


class ArchimedAccountDetails(APIView):
    """Archimed user's account details
    """

    def dispatch(self, request, *args, **kwargs):
        response = super(ArchimedAccountDetails, self).dispatch(request, *args, **kwargs)
        if response.status_code == 200:
            response.data = {'data': response.data, 'err': 0}
        else:
            response.data = {'data': None, 'err': 1, 'err_desc': response.data}
        return response

    def get(self, request, *args, **kwargs):
        logger = logging.getLogger(__name__)
        app_settings = get_app_settings()
        ws_uri = request.build_absolute_uri(
            app_settings.SITE_WEBSERVICES.get_endpoint('account_details'))

        # mellon truncates username to 30 characters
        # thus the passed username must be truncated to 30 characters
        # for searching purpose.
        username = kwargs['username'][:30]

        try:
            user = User.objects.get(username=username)
        except (User.DoesNotExist,):
            return Response('User %s does not exist' % username, status=status.HTTP_404_NOT_FOUND)

        try:
            credentials = UserCredentials.objects.get(user=user)
        except (UserCredentials.DoesNotExist,):
            return Response('User %s is not associated' % username, status=status.HTTP_404_NOT_FOUND)

        login_url = request.build_absolute_uri(
            app_settings.SITE_WEBSERVICES.get_endpoint('login_url'))

        with requests.Session() as session:
            login_info = credentials.to_login_info(decrypt=True)
            login_info = {'username': login_info['#carte'], 'password': login_info['#code']}
            response = session.post(login_url, data=login_info)
            logger.debug("Archimed login response {}".format(response.json()))
            if not response.json()['success']:
                return Response('Authentication failed', status=status.HTTP_401_UNAUTHORIZED)

            content = {
                'codeConfig': '',
                'xslPath': 'Services/LectorShortAccount.xslt'
            }
            response = session.post(ws_uri, json=content)
            logger.debug("Archimed ws response  {}".format(response.json()))
        return Response(response.json())


archimed_account_details = ArchimedAccountDetails.as_view()
