import os
import json
import uuid

import pytest
import mock
from rest_framework.test import APIClient as Client

from mandayejs.applications import Archimed
from utils import create_user, create_credentials

pytestmark = pytest.mark.django_db


def get_base_dir(filename):
    return file(os.path.join(os.path.dirname(__file__), 'data', filename)).read()


class MokcedRequestsResponse(mock.Mock):

    def json(self):
        return json.loads(self.content)


MOCKED_RESPONSES_LIST = [
    MokcedRequestsResponse(content=get_base_dir('archimed_auth_failure_response.json')),
    MokcedRequestsResponse(content=get_base_dir('archimed_auth_success_response.json')),
    MokcedRequestsResponse(content=get_base_dir('archimed_account_detail_response.json'))
]


@mock.patch('mandayejs.applications.views.requests.Session.post')
@mock.patch('mandayejs.applications.get_app_settings')
def test_archimed_ws(mocked_get_app_settings, mocked_requests_post):
    mocked_get_app_settings.return_value = Archimed
    mocked_requests_post.side_effect = MOCKED_RESPONSES_LIST

    user_uuid = uuid.uuid4().hex

    user = create_user(username=user_uuid[:30], password='kevin')

    # test with invalid username
    client = Client()
    client.login(username=user_uuid[:30], password='kevin')
    response = client.get('/_mandaye/ws/account/whatever/')
    assert response.status_code == 404
    assert json.loads(response.content)['err_desc'] == 'User whatever does not exist'
    assert json.loads(response.content)['err'] == 1

    # test with unlinked user
    response = client.get('/_mandaye/ws/account/%s/' % user_uuid)
    assert response.status_code == 404
    assert json.loads(response.content)['err_desc'] == 'User %s is not associated' % user_uuid[:30]
    assert json.loads(response.content)['err'] == 1

    create_credentials(user, {'carte': 'kevin', 'code': 'whatever'})
    # test with wrong credentials
    response = client.get('/_mandaye/ws/account/%s/' % user_uuid)
    assert response.status_code == 401
    assert json.loads(response.content)['err_desc'] == 'Authentication failed'
    assert json.loads(response.content)['err'] == 1

    # test with good credentials
    response = client.get('/_mandaye/ws/account/%s/' % user_uuid)
    assert response.status_code == 200
    assert json.loads(response.content)['data']['message'] == 'Whatever is whatever'
    assert json.loads(response.content)['data']['success'] is True
